[BITS 16]

SETUP		EQU 0x9020

				
sector_end	EQU 18			    ; hardcoded for now
head_end	EQU 1			    ; drive geometry will be taken from BIOS later
	
start_setup:
	mov	ax, SETUP
	mov	es, ax
	mov	ds, ax

	mov	ax, 0x0700
	mov	es, ax
	mov	bx, 0x0000
	call	reset_drive
	call	start_loading


	mov	 cx, 0xFFFF
kill_motor:
	mov	dx,0x3f2
	mov	al,0x0
	out	dx,al
        loop	kill_motor

		
enable_a20:
	cli
	call    wait_keyboard
        mov     al,   0xD1
        out     0x64, al
        call    wait_keyboard
        mov     al,   0xDF
        out     0x60, al
        call    wait_keyboard
	sti

set_gdt: 
	mov ax, 0x8000
        mov es, ax
        mov di, 0x0     
        mov si, gdt	
        mov cx, gdt_end-gdt
	
        cld                
        rep movsb          

        lgdt [gdtr]        
	
	mov	si, a20_ok
	call	write_message

	mov	si, gdt_ok
	call	write_message
	
pic_init:
	cli
	mov	al, 0x11		    ; initialize PICs

	out	0x20, al		    ; 8259_MASTER
	out	0xA0, al		    ; 8259_SLAVE

	mov	al,   0x20		    ; interrupt start 32
	out	0x21, al

	mov	al,   0x28		    ; interrupt start 40
	out	0xA1, al

	mov	al,   0x04		    ; IRQ 2 of 8259_MASTER
	out	0x21, al        

	mov	al,   0x02		    ; to 8259_SLAVE
	out	0xA1, al

	mov	al,   0x01		    ; 8086 Mode
	out	0x21, al
	out	0xA1, al

	mov	al,   0xFF		    ; mask all
	out	0x21, al
	out	0xA1, al
	sti
	
	mov	si, pic_ok
	call	write_message

protected_mode:
	mov	si, pm_ok
	call	write_message
	
	cli
	mov	ax, 0x0001
	lmsw	ax


	mov	ax, 0x10		    ; set selectors to data segment
        mov	ds, ax
        mov	ax, 0x10
        mov	es, ax
        mov	ax, 0x10
        mov	ss, ax
        mov	esp, 0xFFFF	

	push	dword forever
        jmp 0x8:0x7000	

	
forever:				    ; Ooops! Houstin we have a problem 
	jmp forever

	
;;----------- READ_START------------------------|

	
reset_drive:
	
	push	ax
	mov	ah, 0
	int	0x13
	pop	ax
        ret

start_loading:	
	mov	ah, 0x02                    ; ah = read function at int 13h
	mov	al, 0x20		    ; al = the number of sectors to read
        mov	ch, 0                       ; ch = track number
	mov	cl, 0x05                    ; cl = starting sector
        mov	dl, 0x0			    ; dl = drive number
        mov	dh, 0                       ; dh = head number

read_one_sector:
	push	ax
	mov	al, 0x1
	mov	ah, 0x2
	int	0x13
	pop	ax
	
	inc	cl			    ; increment sector value
	dec	al			    ; decrement the number of sectors to read
	add	bx, 0x200		    ; increment the offset
	push	ax
	mov	si, dot
	call	write_message
	pop	ax
	cmp	al, 0		            
	je	load_finished		        
	cmp	cl, sector_end
	je	next_head
	call	reset_drive
	jmp	read_one_sector		    ; read next sector

next_head:
	push	ax
	mov	si, dot
	call	write_message
	pop	ax
	inc	ch
	mov	cl, 0x0
	
	call	reset_drive
	jmp	read_one_sector

load_finished:
	call    reset_drive
	ret

;;----------- READ_END -------------------------|
	
write_message:
	lodsb				    ; al = DS:[SI] and SI++
	or	al, al
	jz	end_message
	mov	ah, 0x0E		    ; teletype Mode
	push	bx
	mov	bx, 0007		    ; white on black attribute
	int	0x10
	pop	bx
	jmp	write_message
	
end_message:
	ret

		
wait_keyboard:	
	in	al, 0x64
	test	al, 2
	jnz	wait_keyboard
	ret


gdtr:
	dw	gdt_end - gdt  - 1	    ; gdt size is one less than the total descriptor sizes
	dd	0x80000			    ; pyhsical address of gdt
	
gdt:
	dd	0x00000000
	dd	0x00000000
	
	dd	0x00000FFF
	dd	0x00C09A00
	
	dd	0x00000FFF
	dd	0x00C09200

gdt_end:



a20_ok		db	13,10,'A20 Line                                                 [ OK ]',13,10,0
gdt_ok		db	'Global Descriptor Table                                  [ OK ]',13,10,0
idt_ok		db	'Interrupt Descriptor Table                               [ OK ]',13,10,0
pic_ok		db	'PIC Initialized                                          [ OK ]',13,10,0
pm_ok		db	'Protected Mode                                           [ OK ]',13,10,0
dot		db	'.', 0




times 1536-($-$$) db 0
